@@ -17,9 +17,9 @@ ERROR awkward_reduce_argmin_complex(
   for (int64_t i = 0;  i < lenparents;  i++) {
     int64_t parent = parents[i];
     if (toptr[parent] == -1  ||
-      (fromptr[i * 2] < fromptr[toptr[parent * 2]]  ||
-        (fromptr[i * 2] == fromptr[toptr[parent * 2]]  &&
-         fromptr[i * 2 + 1] < fromptr[toptr[parent * 2 + 1]]))) {
+      (fromptr[i * 2] < fromptr[toptr[parent] * 2]  ||
+        (fromptr[i * 2] == fromptr[toptr[parent] * 2]  &&
+         fromptr[i * 2 + 1] < fromptr[toptr[parent] * 2 + 1]))) {
       toptr[parent] = i;
     }
   }
