@@ -4,6 +4,7 @@ from __future__ import annotations
 import cupy as cp
 import numpy as np
 import pytest
+from numpy.exceptions import AxisError
 
 import awkward as ak
 from awkward.forms import (
@@ -16,6 +17,13 @@ from awkward.forms import (
 to_list = ak.operations.to_list
 
 
+@pytest.fixture(scope="function", autouse=True)
+def cleanup_cuda():
+    yield
+    cp._default_memory_pool.free_all_blocks()
+    cp.cuda.Device().synchronize()
+
+
 def test_0184_concatenate_operation_records():
     one = ak.highlevel.Array([[1, 2, 3], [None, 4], None, [None, 5]]).layout
     two = ak.highlevel.Array([6, 7, 8]).layout
@@ -262,7 +270,7 @@ def test_1904_drop_none_all_axes():
     assert (
         ak.is_none(array, axis=-2).tolist() == ak.is_none(cuda_array, axis=-2).tolist()
     )
-    with pytest.raises(np.AxisError):
+    with pytest.raises(AxisError):
         ak.drop_none(array, axis=-2).tolist()
         ak.drop_none(cuda_array, axis=-2).tolist()
 
@@ -282,7 +290,7 @@ def test_1904_drop_none_all_axes():
         ak.is_none(array2, axis=-2).tolist()
         == ak.is_none(cuda_array2, axis=-2).tolist()
     )
-    with pytest.raises(np.AxisError):
+    with pytest.raises(AxisError):
         ak.drop_none(array2, axis=-2).tolist()
         ak.drop_none(cuda_array2, axis=-2).tolist()
 
@@ -307,7 +315,7 @@ def test_1914_improved_axis_to_posaxis_is_none():
 
     assert ak.is_none(array, axis=2).tolist() == ak.is_none(cuda_array, axis=2).tolist()
 
-    with pytest.raises(np.AxisError):
+    with pytest.raises(AxisError):
         ak.is_none(array, axis=3)
         ak.is_none(cuda_array, axis=3)
 
@@ -319,7 +327,7 @@ def test_1914_improved_axis_to_posaxis_is_none():
         ak.is_none(array, axis=-2).tolist() == ak.is_none(cuda_array, axis=-2).tolist()
     )
 
-    with pytest.raises(np.AxisError):
+    with pytest.raises(AxisError):
         ak.is_none(array, axis=-3)
         ak.is_none(cuda_array, axis=-3)
 
@@ -353,7 +361,7 @@ def test_1914_improved_axis_to_posaxis_singletons():
         == ak.singletons(cuda_array, axis=2).tolist()
     )
 
-    with pytest.raises(np.AxisError):
+    with pytest.raises(AxisError):
         ak.singletons(array, axis=3)
         ak.singletons(cuda_array, axis=3)
 
@@ -367,7 +375,7 @@ def test_1914_improved_axis_to_posaxis_singletons():
         == ak.singletons(cuda_array, axis=-2).tolist()
     )
 
-    with pytest.raises(np.AxisError):
+    with pytest.raises(AxisError):
         ak.singletons(array, axis=-3)
         ak.singletons(cuda_array, axis=-3)
 
@@ -995,7 +1003,8 @@ def test_2064_fill_none_record_axis_last():
 
 
 def test_2064_fill_none_record_option_outside_record():
-    record = ak.zip({"x": [1, 4], "y": [2, 3]}).mask[[True, False]]
+    record = ak.zip({"x": [1, 4], "y": [2, 3]})
+    record = record.mask[[True, False]]
 
     cuda_record = ak.to_backend(record, "cuda")
 
