@@ -10,6 +10,13 @@ import awkward as ak
 to_list = ak.operations.to_list
 
 
+@pytest.fixture(scope="function", autouse=True)
+def cleanup_cuda():
+    yield
+    cp._default_memory_pool.free_all_blocks()
+    cp.cuda.Device().synchronize()
+
+
 def test_2651_parameter_union():
     layout = ak.contents.IndexedArray(
         ak.index.Index64([0, 1, 2]),
@@ -1190,9 +1197,9 @@ def test_0093_simplify_uniontypes_and_optiontypes_numpyarray_merge():
             cuda_two = ak.to_backend(two, "cuda", highlevel=False)
 
             cuda_three = cuda_one._mergemany([cuda_two])
-            assert ak.to_numpy(cuda_three).dtype == np.dtype(
-                z
-            ), f"{x} {y} {z} {ak.to_numpy(cuda_three).dtype.type}"
+            assert ak.to_numpy(cuda_three).dtype == np.dtype(z), (
+                f"{x} {y} {z} {ak.to_numpy(cuda_three).dtype.type}"
+            )
             assert to_list(cuda_three) == to_list(
                 np.concatenate([ak.to_numpy(cuda_one), ak.to_numpy(two)])
             )
